#!/bin/sh

. /etc/utils/shell-utils.sh

DEBUG=0

WIFI_ID_FILE=/ram/wifi/wifi-id.lock
WIFI_CURR_DRIVER_FILE=/ram/wifi/curr_driver

WIFI_DRIVER_MODULE_NAME="rt5370sta"
WIFI_DRIVER_MT_MODULE_NAME="mt7601Usta"
WIFI_DRIVER_MT7610_MODULE_NAME="mt7610u_sta"
WIFI_DRIVER_REALTEK_MODULE_NAME="8188eu"

load_module()
{
    log "Load kernel module. wifi_driver='$wifi_driver'"

    if [ "$WIFI_DRIVER_MT_MODULE_NAME" == "${wifi_driver}" ]; then
	log "Loading '$WIFI_DRIVER_MT_MODULE_NAME' modules..."
	insmod /lib/modules/mt7601Usta.ko
        return;
    fi

    if [ "$WIFI_DRIVER_MT7610_MODULE_NAME" == "${wifi_driver}" ]; then
	log "Loading '$WIFI_DRIVER_MT7610_MODULE_NAME' modules..."
	insmod /lib/modules/mt7610u_sta.ko
        return;
    fi

    if [ "$WIFI_DRIVER_REALTEK_MODULE_NAME" == "${wifi_driver}" ]; then
	log "Loading '$WIFI_DRIVER_REALTEK_MODULE_NAME' modules..."
	insmod /lib/modules/8188eu.ko
        return;
    fi

    if [ "$WIFI_DRIVER_MODULE_NAME" == "${wifi_driver}" ]; then
	log "Loading '$WIFI_DRIVER_MODULE_NAME' modules..."
	insmod /lib/modules/rt5370sta.ko
	echo 2001 3c1e >> /sys/bus/usb/drivers/rt2870/new_id
	echo 0b05 17e8 >> /sys/bus/usb/drivers/rt2870/new_id
	echo 0b05 179d >> /sys/bus/usb/drivers/rt2870/new_id
        return;
    fi

    log "Could not load any modules!!! Ignoring load..."
}

unload_module()
{
    wifi_driver=`cat ${WIFI_CURR_DRIVER_FILE}`

    log "Unload kernel module for driver '${wifi_driver}'...'"

    if [ "$WIFI_DRIVER_MT_MODULE_NAME" == "${wifi_driver}" ]; then
	log "Unloading '$WIFI_DRIVER_MT_MODULE_NAME' modules..."
	rmmod $WIFI_DRIVER_MT_MODULE_NAME
        return;
    fi

    if [ "$WIFI_DRIVER_MT7610_MODULE_NAME" == "${wifi_driver}" ]; then
	log "Unloading '$WIFI_DRIVER_MT7610_MODULE_NAME' modules..."
	rmmod $WIFI_DRIVER_MT7610_MODULE_NAME
        return;
    fi

    if [ "$WIFI_DRIVER_REALTEK_MODULE_NAME" == "${wifi_driver}" ]; then
	log "Unloading '$WIFI_DRIVER_REALTEK_MODULE_NAME' modules..."
	rmmod $WIFI_DRIVER_REALTEK_MODULE_NAME
        return;
    fi

    if [ "$WIFI_DRIVER_MODULE_NAME" == "${wifi_driver}" ]; then
	log "Unloading '$WIFI_DRIVER_MODULE_NAME' modules..."
	rmmod $WIFI_DRIVER_MODULE_NAME
        return;
    fi

    log "Could not unload any modules!!! Ignoring unload..."
}

detect_module()
{
    # get correct idProduct here. we cant really rely on one in udev event because there should be empty value in many cases
    USB_PRODUCT_ID=`udevinfo -a -p ${DEVPATH} | grep idProduct | head -1 | sed -n 's/^[ ]*ATTRS{idProduct}==\"\(.*\)\"$/\1/p'`

    log "Detect module type. USB_PRODUCT='${USB_PRODUCT}', USB_VENDOR_ID='${USB_VENDOR_ID}', USB_PRODUCT_ID='${USB_PRODUCT_ID}'"

  if [ "${USB_VENDOR_ID}" == "0bda" ]; then
    log "found vendor = Realtek"
      export wifi_driver=$WIFI_DRIVER_REALTEK_MODULE_NAME
      retVal=1
      return
  fi


  if [ "${USB_VENDOR_ID}" == "148f" ]; then
    log "found vendor = Ralink Technology, Corp."
    if [ "${USB_PRODUCT_ID}" == "7601" ]; then
      log "found product = MediaTek mt7601u"
      export wifi_driver=$WIFI_DRIVER_MT_MODULE_NAME
      retVal=1
      return
    fi
  fi

  if [ "${USB_VENDOR_ID}" == "0b05" ]; then
    log "found vendor = ASUS(0x0b05)"
    if [ "${USB_PRODUCT_ID}" == "1784" ]; then
      log "found product = USB-N13(0x1784)"
      export wifi_driver=$WIFI_DRIVER_MODULE_NAME
      retVal=1
      return
    fi
  fi

  if [ "${USB_VENDOR_ID}" == "07d1" ]; then
    log "found vendor = D-Link System (0x07d1)"
    if [ "${USB_PRODUCT_ID}" == "3c0d" ]; then
      log "found product = DWA-125 Wireless 150 USB Adapter"
      export wifi_driver=$WIFI_DRIVER_MODULE_NAME
      retVal=1
      return
    fi
  fi

  if [ "${USB_VENDOR_ID}" == "07d1" ]; then
    log "found vendor = D-Link System (0x07d1)"
    if [ "${USB_PRODUCT_ID}" == "3c16" ]; then
      log "found product = DWA-125 Wireless N 150 Adapter (rev.A2)"
      export wifi_driver=$WIFI_DRIVER_MODULE_NAME
      retVal=1
      return
    fi
  fi

  if [ "${USB_VENDOR_ID}" == "2001" ]; then
    log "found vendor = D-Link System (0x07d1)"
    if [ "${USB_PRODUCT_ID}" == "3c19" ]; then
      log "found product = DWA-125 Wireless N 150 Adapter (rev.A3)"
      export wifi_driver=$WIFI_DRIVER_MODULE_NAME
      retVal=1
      return
    fi
  fi

  if [ "${USB_VENDOR_ID}" == "7392" ]; then
    log "found vendor = Edimax (0x7392)"
    if [ "${USB_PRODUCT_ID}" == "7711" ]; then
      log "found product = EW-7711UAn (0x7711)"
      export wifi_driver=$WIFI_DRIVER_MODULE_NAME
      retVal=1
      return
    fi
  fi
  if [ "${USB_VENDOR_ID}" == "148f" ]; then
    log "found vendor = Ralink Technology, Corp. (0x148f)"
    if [ "${USB_PRODUCT_ID}" == "3070" ] || [ "${USB_PRODUCT_ID}" == "5370" ]; then
      log "found product = RT2870/RT3070 Wireless Adapter (0x3070)"
      export wifi_driver=$WIFI_DRIVER_MODULE_NAME
      retVal=1
      return
    fi
  fi

  if [ "${USB_VENDOR_ID}" == "1737" ]; then
    log "found vendor = Ralink Technology, Corp. (0x148f)"
    if [ "${USB_PRODUCT_ID}" == "0079" ] || [ "${USB_PRODUCT_ID}" == "0079" ]; then
      log "WUSB600N"
      export wifi_driver=$WIFI_DRIVER_MODULE_NAME
      retVal=1
      return
    fi
  fi

  if [ "${USB_VENDOR_ID}" == "7392" ]; then
    log "found vendor = Ralink Technology, Corp. (0x148f)"
    if [ "${USB_PRODUCT_ID}" == "7733" ] || [ "${USB_PRODUCT_ID}" == "7733" ]; then
      log "WUSB600N"
      export wifi_driver=$WIFI_DRIVER_MODULE_NAME
      retVal=1
      return
    fi
  fi

  if echo $4 | grep -i "ralink" > /dev/null ; then
    log "found vendor = Ralink Technology, Corp. (common case)"
    export wifi_driver=$WIFI_DRIVER_MODULE_NAME
    retVal=1
    return
  fi

    log "going to test mfct = ${USB_MANUFACTURER}"

    if echo "${USB_MANUFACTURER}" | grep -i "MediaTek" > /dev/null ; then
        log "Found device of 'MediaTek' manufacturer. Trying it..."
#        if [ "${USB_PRODUCT_ID}" == "7610" ]; then
            log "Found product family MediaTek/mt7610"
            export wifi_driver=$WIFI_DRIVER_MT7610_MODULE_NAME
            retVal=1
            return
#        fi
    fi

    if echo "${USB_MANUFACTURER}" | grep -i "ralink" > /dev/null ; then
        log "Found device of 'Ralink Technology' manufacturer. Trying it..."
        export wifi_driver=$WIFI_DRIVER_MODULE_NAME
        retVal=1
        return
    fi

    if echo "${USB_PRODUCT}" | egrep -i "(802.11)|(WLAN)|(80211)" > /dev/null ; then
        log "Found '802.11n WLAN Adapter' product family device. Trying it..."
        export wifi_driver=${WIFI_DRIVER_MODULE_NAME}
        retVal=1
        return
    fi

}


# EXECUTION POINT

log "Action='$UDEV_ACTION', USB_PRODUCT='${USB_PRODUCT}', DEVPATH='${DEVPATH}'"

if ! echo ${DEVPATH} | egrep -i "^.+:[0-9].[0-9]" > /dev/null ; then
    log "Ignoring device."
    exit 0
fi

export wifi_bus_id=${DEVPATH}

case $UDEV_ACTION in
    add)
	log "Cmd: ADD."

	if [ -f $WIFI_CURR_DRIVER_FILE ]; then
    	    log "WIFI is already activated. Exiting..."
	    exit 1;
	fi

	detect_module

	if [ "$wifi_driver" == "" ]; then
    	    log "Unsupported product. Exiting..."
	    exit 1;
	fi

	echo "${wifi_bus_id}" > $WIFI_ID_FILE
	echo "${wifi_driver}" > $WIFI_CURR_DRIVER_FILE

	load_module

	log "Load OK"

	sleep 1

	/usr/sbin/wpa_supplicant -B -Dwext -iwlan0 -c/etc/wpa_supplicant/wpa_supplicant.conf > /dev/null 2>&1

	log "Supplicant started OK"

	/etc/init.d/wifi_start_wa.sh &
    ;;
    remove)
	log "Cmd: REMOVE"

	curr_wifi_id_lock=`cat $WIFI_ID_FILE`

	if [ "${wifi_bus_id}" == "$curr_wifi_id_lock" ]; then
	    unload_module
    	    rm -f $WIFI_ID_FILE
    	    rm -f $WIFI_CURR_DRIVER_FILE
	    killall wpa_supplicant
	    /etc/init.d/udhcpcd stop wlan0
	else
    	    log "Its not the WiFi dongle we allowed to deal with"
	fi

    ;;
    stop)
	log "Cmd: STOP"

	killall -9 wpa_supplicant

	log "Supplicant KILLED"
    ;;
    start)
	log "Cmd: START"

	if [ ! -f $WIFI_CURR_DRIVER_FILE ]; then
    	    log "WIFI isn't activated. Exiting..."
	    exit 1;
	fi

	/usr/sbin/wpa_supplicant -B -Dwext -iwlan0 -c/etc/wpa_supplicant/wpa_supplicant.conf > /dev/null 2>&1

	log "Supplicant started OK"
    ;;
esac

log "DONE."

exit 0
